"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TionMagicAirStation = void 0;
const framework_1 = require("../../homebridge/framework");
const base_1 = require("./base");
const state_1 = require("../state");
const command_1 = require("../command");
class TionMagicAirStation extends base_1.TionDeviceBase {
    constructor() {
        super(...arguments);
        this.co2Level = 0;
        this.temperature = 0;
        this.humidity = 0;
        this.backlight = false;
    }
    addEventHandlers(accessory) {
        accessory.reachable = true;
        if (this.accessories.find(a => a.UUID === accessory.UUID)) {
            return;
        }
        this.accessories.push(accessory);
        const co2Sensor = accessory.getService(this.serviceRegistry.CarbonDioxideSensor);
        if (co2Sensor) {
            co2Sensor
                .getCharacteristic(this.characteristicRegistry.CarbonDioxideDetected)
                .on('get', callback => this.getState(callback, () => this.carbonDioxideDetected()));
            co2Sensor
                .getCharacteristic(this.characteristicRegistry.CarbonDioxideLevel)
                .on('get', callback => this.getState(callback, () => this.co2Level));
        }
        const temperatureSensor = accessory.getService(this.serviceRegistry.TemperatureSensor);
        if (temperatureSensor) {
            temperatureSensor
                .getCharacteristic(this.characteristicRegistry.CurrentTemperature)
                .on('get', callback => this.getState(callback, () => this.temperature));
        }
        const humiditySensor = accessory.getService(this.serviceRegistry.HumiditySensor);
        if (humiditySensor) {
            humiditySensor
                .getCharacteristic(this.characteristicRegistry.CurrentRelativeHumidity)
                .on('get', callback => this.getState(callback, () => this.humidity));
        }
        const backlightSwitch = accessory.getService(this.serviceRegistry.Switch);
        if (backlightSwitch) {
            backlightSwitch
                .getCharacteristic(this.characteristicRegistry.On)
                .on('get', callback => this.getState(callback, () => this.backlight))
                .on('set', (value, callback) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (!this.isOnline) {
                        this.log.error(`Device ${this.name} (${this.id}) not reachable`);
                        return callback('Not reachable');
                    }
                    value = Boolean(value);
                    if (value !== this.backlight) {
                        yield this.setState(command_1.CommandType.Settings, {
                            backlight: value ? 1 : 0,
                        });
                    }
                    this.backlight = value;
                    callback();
                }
                catch (err) {
                    this.log.error(err.message || err);
                    callback(err.message || err);
                }
            }));
        }
    }
    updateState(state) {
        this.parseState(state);
        this.accessories.forEach(accessory => {
            accessory.reachable = this.isOnline;
            const co2Sensor = accessory.getService(this.serviceRegistry.CarbonDioxideSensor);
            if (co2Sensor) {
                co2Sensor.setCharacteristic(this.characteristicRegistry.CarbonDioxideDetected, this.carbonDioxideDetected());
                co2Sensor.setCharacteristic(this.characteristicRegistry.CarbonDioxideLevel, this.co2Level);
            }
            const temperatureSensor = accessory.getService(this.serviceRegistry.TemperatureSensor);
            if (temperatureSensor) {
                temperatureSensor.setCharacteristic(this.characteristicRegistry.CurrentTemperature, this.temperature);
            }
            const humiditySensor = accessory.getService(this.serviceRegistry.HumiditySensor);
            if (humiditySensor) {
                humiditySensor.setCharacteristic(this.characteristicRegistry.CurrentRelativeHumidity, this.humidity);
            }
            const backlightSwitch = accessory.getService(this.serviceRegistry.Switch);
            if (backlightSwitch) {
                backlightSwitch.setCharacteristic(this.characteristicRegistry.On, this.backlight);
            }
        });
    }
    parseState(state) {
        var _a, _b, _c, _d;
        const { device } = this.findDeviceInState(state);
        if (!device) {
            return false;
        }
        this.isOnline = device.is_online;
        this.co2Level = ((_a = device.data) === null || _a === void 0 ? void 0 : _a.co2) || 0;
        this.temperature = ((_b = device.data) === null || _b === void 0 ? void 0 : _b.temperature) || 0;
        this.humidity = ((_c = device.data) === null || _c === void 0 ? void 0 : _c.humidity) || 0;
        this.backlight = Boolean((_d = device.data) === null || _d === void 0 ? void 0 : _d.backlight);
        return true;
    }
    carbonDioxideDetected() {
        return this.co2Level > this.config.co2Threshold ? 1 : 0;
    }
}
exports.TionMagicAirStation = TionMagicAirStation;
//# sourceMappingURL=station.js.map